Natural

Enum Natural 

pub enum Natural {
    Zero,
    Nonzero(Composite),
}
Natural number, either a Composite or Zero

m = prod p_k^{n_k} OR 0

Note this struct does NOT implement Divisible. That trait is meant for numbers that can be uniquely decomposed into Primes. This is not the case, since Natural can be zero.

Variants



Zero

The natural number 0



Nonzero(Composite)

A nonzero natural number

Implementations

Source

impl Natural

pub fn gcd(&self, other: &Self) -> Self

Greatest common divisor

use adic::divisible::Natural;
let n1 = Natural::from(6);
let n2 = Natural::from(15);
assert_eq!(Natural::from(3), n1.gcd(&n2));
Panics

Panics if either self or other are zero

pub fn lcm(&self, other: &Self) -> Self

Least common multiple

use adic::divisible::Natural;
let n1 = Natural::from(6);
let n2 = Natural::from(15);
assert_eq!(Natural::from(30), n1.lcm(&n2));
assert_eq!(Natural::from(0), n1.lcm(&Natural::from(0)));