Valuation

Enum Valuation 

pub enum Valuation<F>
where F: ValuationRing,
{ PosInf, Finite(F), }
Represents valuations of ultrametric numbers

In the digital representation, this is the number of digits between decimal point and first nonzero digit, possibly negative. This struct can also be used to represent certainty. E.g.

let z = ZAdic::new_approx(5, 6, vec![0, 0, 3, 1, 2, 4]);
assert_eq!(Valuation::Finite(2), z.valuation());
assert_eq!(Valuation::Finite(6), z.certainty());
assert_eq!(Valuation::Finite(0), ZAdic::empty(5).valuation());
assert_eq!(Valuation::Finite(0), ZAdic::empty(5).certainty());
assert_eq!(Valuation::PosInf, UAdic::new(5, vec![]).valuation());
assert_eq!(Valuation::PosInf, UAdic::new(5, vec![]).certainty());
assert_eq!(Valuation::Finite(0), QAdic::new(UAdic::new(5, vec![1, 2]), 0).valuation());
assert_eq!(Valuation::Finite(2), QAdic::new(UAdic::new(5, vec![1, 2]), 2).valuation());
assert_eq!(Valuation::Finite(-2), QAdic::new(UAdic::new(5, vec![1, 2]), -2).valuation());
assert_eq!(Valuation::Finite(-1), QAdic::new(UAdic::new(5, vec![0, 2]), -2).valuation());
assert_eq!(Valuation::PosInf, QAdic::new(UAdic::zero(5), -2).valuation());

Variants



PosInf

Positive infinity, e.g. for the size of zero



Finite(F)

Finite valuation

Implementations

Source

impl<F> Valuation<F>
where F: ValuationRing,

pub fn finite(&self) -> Option<F>

Return finite value if Finite and None if PosInf

pub fn is_finite(&self) -> bool

Is the valuation finite

pub fn convert<G>(self) -> Result<Valuation<G>, G::Error>
where G: ValuationRing + TryFrom<F>,

Convert from one valuation to another

Errors

Error if the conversion attempt fails