QAdic

Struct QAdic 

pub struct QAdic<A>
where A: AdicInteger,
{ /* private fields */ }
Fractional adic number

The struct holds an adic integer and a valuation. Digitally, there are -valuation digits to the right of the decimal. With this, you can represent any adic number.

The adic integer is generic and so can be e.g.

let twenty_three_and_11_25 = QAdic::new(UAdic::new(5, vec![1, 2, 3, 4]), Valuation::Finite(-2));
assert_eq!("43.21_5", twenty_three_and_11_25.to_string());
let fifty = QAdic::new(UAdic::new(5, vec![0, 2]), 1);
assert_eq!("200._5", fifty.to_string());
let neg_one_tenth = QAdic::new(EAdic::new_repeating(5, vec![], vec![2]), -1);
assert_eq!("(2).2_5", neg_one_tenth.to_string());

assert_eq!(
    QAdic::new(UAdic::new(5, vec![1, 2, 4, 1, 1]), -2),
    QAdic::new(UAdic::new(5, vec![1, 2, 3, 4]), -2) + QAdic::new(UAdic::new(5, vec![1, 2]), 0)
);
assert_eq!(
    QAdic::new(UAdic::new(5, vec![2, 2]), -3),
    QAdic::new(UAdic::new(5, vec![3]), -2) * QAdic::new(UAdic::new(5, vec![4]), -1)
);

assert_eq!(Ratio::new(1, 1), QAdic::new(UAdic::new(5, vec![4, 1, 3, 2]), 0).norm());
assert_eq!(Ratio::new(1, 25), QAdic::new(UAdic::new(5, vec![4, 1, 3, 2]), 2).norm());
assert_eq!(Ratio::new(25, 1), QAdic::new(UAdic::new(5, vec![4, 1, 3, 2]), -2).norm());

This struct represents adic numbers as base-p digital expansions, with a possibly-infinite number of digits to the left of a decimal point and a finite number of digits to the right.

AdicIntegers are similar, but without digits to the right of the decimal. QAdics can represent all rational numbers as well as many irrational (distinct from the real number irrationals). Using the p-adic norm, these numbers have valuation -inf < v <= inf, i.e. |x| = p^(-v).

let neg_one = -EAdic::one(5);
let neg_one_fifth = QAdic::new(neg_one.clone(), -1);
let neg_one_twenty_fifth = QAdic::new(neg_one.clone(), -2);
let sqrt_neg_one = ZAdic::new_approx(5, 6, vec![2, 1, 2, 1, 3, 4]);
assert_eq!("...431212._5", sqrt_neg_one.to_string());
let sqrt_neg_one_twenty_fifth = QAdic::new(sqrt_neg_one.clone(), -1);
assert_eq!("...43121.2_5", sqrt_neg_one_twenty_fifth.to_string());
assert_eq!(
    Ok(Variety::new(vec![sqrt_neg_one.clone(), -sqrt_neg_one.clone()])),
    neg_one.nth_root(2, 6)
);
assert!(neg_one_fifth.nth_root(2, 5).is_ok_and(|variety| variety.is_empty()));
assert_eq!(
    Ok(Variety::new(vec![sqrt_neg_one_twenty_fifth.clone(), -sqrt_neg_one_twenty_fifth])),
    neg_one_twenty_fifth.nth_root(2, 5)
);

https://en.wikipedia.org/wiki/P-adic_number

Panics

Many methods will panic if a provided prime p is not prime or digits are outside of [0, p).

Implementations

Source

impl<A> QAdic<A>
where A: AdicInteger,

pub fn new<V>(adic_int: A, valuation: V) -> Self
where V: Into<Valuation<isize>>,

Create an adic number with the given digits and valuation

pub fn frac_and_int(&self) -> (QAdic<UAdic>, A)

Split QAdic into fraction (as QAdic<UAdic>) and integer (as A)

let r = EAdic::new_repeating(7, vec![1, 2], vec![3, 4, 5]);
assert_eq!("(543)21._7", r.to_string());
let q = QAdic::new(r, -6);
assert_eq!("(354).354321_7", q.to_string());
let (q_frac, q_int) = q.frac_and_int();
assert_eq!("0.354321_7", q_frac.to_string());
assert_eq!("(354)._7", q_int.to_string());
pub fn from_integer(adic_int: A) -> Self

Create an adic number with the given digits and zero valuation

pub fn try_into_integer(self) -> AdicResult<A>

Try to convert into an AdicInteger, returning error if there are fractional digits

let r = EAdic::new_repeating(7, vec![1, 2], vec![3, 4, 5]);
assert_eq!("(543)21._7", r.to_string());
let q = QAdic::new(r.clone(), 3);
assert_eq!("(543)21000._7", q.to_string());
let q_int = q.try_into_integer();
assert_eq!(Ok("(543)21000._7".to_string()), q_int.map(|a| a.to_string()));
let q = QAdic::new(r, -3);
assert_eq!("(354).321_7", q.to_string());
let q_int = q.try_into_integer();
assert_eq!(Err(AdicError::AdicIntegerExpected), q_int);
pub fn nth_root( &self, n: u32, precision: isize, ) -> AdicResult<Variety<QAdic<ZAdic>>>
where Self: Into<QAdic<ZAdic>>,

Calculate the n-th root, to precision digits, using Hensel lifting.

This is a specific case of Polynomial::variety, for the polynomial f(x) = x^n - a = 0.

If n has a factor of p, then the algorithm is more complicated because you have to take into account more digits.

7-adic sqrt(1/98) has two solutions, starting with 3 and with 4

let seven_adic_2_49 = QAdic::new(UAdic::new(7, vec![2]), -2);
let variety = seven_adic_2_49.nth_root(2, 6).unwrap();
let expected = Variety::new(vec![
    QAdic::new(ZAdic::new_approx(7, 7, vec![3, 1, 2, 6, 1, 2, 1]), -1),
    QAdic::new(ZAdic::new_approx(7, 7, vec![4, 5, 4, 0, 5, 4, 5]), -1),
]);
assert_eq!(expected, variety);
assert_eq!("variety(...121621.3_7, ...545045.4_7)", variety.to_string());
Errors
  1. QAdic’s certainty is not high enough for desired precision
  2. n == 0
Panics

Panics if certainty does not behave as expected

pub fn num_nth_roots(&self, n: u32) -> AdicResult<usize>
where Self: Into<QAdic<ZAdic>>,

Return the number of n-th roots of this QAdic

let two_49ths = QAdic::<EAdic>::primed_from(7, Rational32::new(2, 49));
assert_eq!(Ok(0), two_49ths.num_nth_roots(0));
assert_eq!(Ok(1), two_49ths.num_nth_roots(1));
assert_eq!(Ok(2), two_49ths.num_nth_roots(2));
assert_eq!(Ok(0), two_49ths.num_nth_roots(3));
assert_eq!(Ok(0), two_49ths.num_nth_roots(4));
assert_eq!(Ok(0), two_49ths.num_nth_roots(5));
assert_eq!(Ok(0), two_49ths.num_nth_roots(6));
assert_eq!(Ok(0), two_49ths.num_nth_roots(7));
Errors

Errors if rootfinding encounters problems, e.g. heavily degenerate roots

Source

impl QAdic<ZAdic>

pub fn empty<P, V>(p: P, v: V) -> Self
where P: Into<Prime>, V: Into<Valuation<isize>>,

Create the empty QAdic<ZAdic> with valuation v